package se.gawell.kvtstore.core.riak;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import se.gawell.kvtstore.core.DataStoreException;
import se.gawell.kvtstore.core.Deserializer;
import se.gawell.kvtstore.core.KeyTimeValueStore;
import se.gawell.kvtstore.core.Serializer;
import se.gawell.kvtstore.core.TimeProvider;

import com.basho.riak.client.http.RiakClient;
import com.basho.riak.client.http.RiakObject;
import com.basho.riak.client.http.request.RequestMeta;
import com.basho.riak.client.http.response.FetchResponse;
import com.basho.riak.client.http.response.IndexResponse;
import com.basho.riak.client.http.response.StoreResponse;

@Component
public class RiakKeyTimeValueStore implements KeyTimeValueStore {

	public static final String CREATION_TIME_INDEX = "creation_time_index_bin";
	private final RiakClient riakClient;
	private final Serializer serializer;
	private final Deserializer deserializer;
	private final TimeProvider timeProvider;
	private final TimeFormatter timeFormatter;

	@Autowired
	public RiakKeyTimeValueStore(RiakClient riakClient, Serializer serializer, Deserializer deserializer,
			TimeProvider timeProvider, TimeFormatter timeFormatter) {
		super();
		this.riakClient = riakClient;
		this.serializer = serializer;
		this.deserializer = deserializer;
		this.timeProvider = timeProvider;
		this.timeFormatter = timeFormatter;
	}

	@Override
	public void put(Serializable primaryKey, Object object) {
		String serializedObject = serializer.serialize(object);
		String key = primaryKey.toString();
		String bucketNameForClass = getBucketNameForClass(object.getClass());
		
		RiakObject riakObject = new RiakObject(bucketNameForClass, key, serializedObject.getBytes());
		Date now = timeProvider.now();
		riakObject.addIndex(CREATION_TIME_INDEX, timeFormatter.format(now));
		RequestMeta requestMeta = new RequestMeta();
		requestMeta.setHeader("Content-Type", "application/json");
		StoreResponse storeResponse = riakClient.store(riakObject, requestMeta );
		if (!storeResponse.isSuccess()) {
			throw new DataStoreException(storeResponse.getHttpResponse().toString() + ": " + storeResponse.getBodyAsString());
		}
	}

	private String getBucketNameForClass(Class<? extends Object> type) {
		return type.getCanonicalName();
	}

	@Override
	public <type> type getById(Serializable primaryKey, Class<? extends Object> type) {
		String key = primaryKey.toString();
		FetchResponse fetch = riakClient.fetch(getBucketNameForClass(type), key);
		String serializedObject = fetch.getObject().getValue();
		return deserializer.deserialize(serializedObject, type);
	}

	@SuppressWarnings("unchecked")
	@Override
	public <type> List<type> getByTimeInterval(Date startOfIntervalInclusive, Date endOfIntervalExclusive,
			Class<? extends Object> type) {
		String startOfIntervalInclusiveAsString = timeFormatter.format(startOfIntervalInclusive);
		String endOfIntervalExclusiveAsString = timeFormatter.format(endOfIntervalExclusive);
		IndexResponse index = riakClient.index(getBucketNameForClass(type), CREATION_TIME_INDEX, startOfIntervalInclusiveAsString,
				endOfIntervalExclusiveAsString);
		if (index.isError()) {
			throw new DataStoreException("Could not read indexes from riak. " + index.getHttpResponse() + ". Message body: " + createStringFromByteArray(index.getBody()));
		}
		ArrayList<type> list = new ArrayList<type>(index.getKeys().size());
		for (String key: index.getKeys()) {
			list.add((type) getById(key, type));
		}
		return list;
	}

	private String createStringFromByteArray(byte[] bytes) {
		if (bytes != null) {
			return new String(bytes);
		} else {
			return null;
		}
	}

}
